/*
 * Knapsack - simulated annealing
 * Tomislav Novak
 */
#include <algorithm>
#include <cstdio>
#include <cstdlib>
#include <cassert>
#include <climits>
#include <ctime>
#include <cmath>
using namespace std;

#define MAXN 1000000
#define INF  INT_MAX/2

struct k {
    int cost, value;
};

k      knapsack[MAXN];
bool   stanje[MAXN];
int    cost, value;
int    n, W, niter;
double T, alfa;

int irand(int n) {
    return (int)((double)n * (rand() / (RAND_MAX + 1.0)));
}

void calc(int &cost, int &value) {
    cost = value = 0;
    for (int i = 0; i < n; ++i) {
        cost  += knapsack[i].cost  * stanje[i];
        value += knapsack[i].value * stanje[i];
    }

    if (cost > W)
        value = -INF;
}

int main(int argc, char *argv[]) {
    if (argc != 4) {
        fprintf(stderr, "Koristenje: %s niter temperatura alfa\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    niter = atoi(argv[1]);
    sscanf(argv[2], "%lf", &T);
    sscanf(argv[3], "%lf", &alfa);

    scanf("%d%d", &W, &n);
    assert(n <= MAXN);

    for (int i = 0; i < n; ++i)
        scanf("%d%d", &knapsack[i].cost, &knapsack[i].value);

    srand(time(NULL));

    int 
        cost  = 0,
        value = 0;

    // pocetno stanje - slucajno dopustivo
    for (int i = 0; i < n; ++i) {
        int q = irand(n);

        if (stanje[q] == 0 && cost+knapsack[q].cost <= W)
            stanje[q] = 1, cost += knapsack[q].cost;
        else if (stanje[q] == 1)
            stanje[q] = 0, cost -= knapsack[q].cost;
    }

    for (int i = 0; i < n; ++i)
        fprintf(stderr, "%d ", stanje[i]);

    calc(cost, value);
    fprintf(stderr, "\tCOST = %d, VALUE = %d\n", cost, value);

    int best = value;
    while (niter--) {
        int c, v, k = irand(n), ok = 0;
        
        if (!stanje[k] && cost+knapsack[k].cost > W) {
            T *= alfa;
            continue;
        }
            

        // promijeni stanje proizvoljnom bitu
        stanje[k] ^= 1; calc(c, v); stanje[k] ^= 1;

        if (v > value) // ako je novo stanje bolje, prelazimo u njega
            value = v, cost = c, ok = 1;
        else {
            // ako je losije, prelazimo s odredjenom vjerojatnoscu
            double p = exp((v-value)/T);
            fprintf(stderr, "VJEROJATNOST prelaska u stanje s VALUE=%d: %lf\n",
                    v, p);
            if (p - rand()/(double)RAND_MAX > 10e-9)
                value = v, cost = c, ok = 1;
        }

        if (ok)
            stanje[k] ^= 1;

        best = max(best, value);

        for (int i = 0; i < n; ++i)
            fprintf(stderr, "%d ", stanje[i]);
        fprintf(stderr, "\tCOST = %d, VALUE = %d, TEMP=%lf\n", cost, value, T);

        T *= alfa; // temperatura se postupno smanjuje
    }

    printf("%d\n", best);

    return EXIT_SUCCESS;
}
