/*
 * Knapsack - tabu search
 * Tomislav Novak
 */
#include <algorithm>
#include <vector>
#include <deque>
#include <cstdio>
#include <cstdlib>
#include <cassert>
#include <climits>
#include <ctime>
#include <cmath>
using namespace std;

#define BIG  1000000000.0
#define MAXN 1000000
#define MAXL 1000
#define INF  INT_MAX/2

struct k {
    int cost, value;
};

deque <int> tabu;

k    knapsack[MAXN];
bool stanje[MAXN];
int  cost, value, best_value;
int  n, W, niter, tabu_size;

int irand(int n) {
    return (int)((double)n * (rand() / (RAND_MAX + 1.0)));
}

bool trazi_tabu(int u) {
    for (deque<int>::iterator it = tabu.begin(); it != tabu.end(); ++it)
        if (*it == u)
            return true;

    return false;
}

inline void dodaj_tabu(int u) {
    tabu.push_back(u);
    if ((int)tabu.size() > tabu_size)
        tabu.pop_front();
}

void calc(int &cost, int &value) {
    cost = value = 0;
    for (int i = 0; i < n; ++i) {
        cost  += knapsack[i].cost  * stanje[i];
        value += knapsack[i].value * stanje[i];
    }

//    if (cost > W)
//        value = -INF;
}

int main(int argc, char *argv[]) {
    if (argc != 3) {
        fprintf(stderr, "Koristenje: %s niter tabu_size\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    niter     = atoi(argv[1]);
    tabu_size = atoi(argv[2]);

    scanf("%d%d", &W, &n);
    assert(n <= MAXN);

    srand(time(NULL));

    for (int i = 0; i < n; ++i)
        scanf("%d%d", &knapsack[i].cost, &knapsack[i].value);

    // pocetno stanje - slucajno generirano dopustivo
    for (int i = 0; i < n; ++i) {
        int q = irand(n);

        if (stanje[q] == 0 && cost+knapsack[q].cost <= W)
            stanje[q] = 1, cost += knapsack[q].cost;
        else if (stanje[q] == 1)
            stanje[q] = 0, cost -= knapsack[q].cost;
    }

    for (int i = 0; i < n; ++i)
        fprintf(stderr, "%d ", stanje[i]);

    calc(cost, value);
    fprintf(stderr, "\tCOST = %d, VALUE = %d\n\n", cost, value);

    int best_value = value;
    while (niter--) {
        double best = 0.0;
        bool   mogu_povecati = true;
        int    b_ind = -1;

        for (int i = 0; i < n; ++i)
            if (stanje[i] == 0 && !trazi_tabu(i) && 
                cost+knapsack[i].cost <= W &&
                knapsack[i].value/(double)knapsack[i].cost > best) {
                b_ind = i;
                best  = knapsack[i].value/(double)knapsack[i].cost;
            }

        if (b_ind == -1)
            mogu_povecati = false;

        if (mogu_povecati) {
            cost  += knapsack[b_ind].cost;
            value += knapsack[b_ind].value;

            best_value = max(best_value, value);

            stanje[b_ind] = 1;
            dodaj_tabu(b_ind);

            for (int i = 0; i < n; ++i)
                fprintf(stderr, "%d ", stanje[i]);
            fprintf(stderr, "\tCOST = %d, VALUE = %d\n", cost, value);

            continue;
        }

        best = BIG;
        for (int i = 0; i < n; ++i)
            if (stanje[i] && !trazi_tabu(i) && 
                knapsack[i].value/(double)knapsack[i].cost < best) {
                b_ind = i;
                best  = knapsack[i].value/(double)knapsack[i].cost;
            }

        if (b_ind == -1)
            break;

        stanje[b_ind] = 0;
        dodaj_tabu(b_ind);

        cost  -= knapsack[b_ind].cost;
        value -= knapsack[b_ind].value;

        best_value = max(best_value, value);

        for (int i = 0; i < n; ++i)
            fprintf(stderr, "%d ", stanje[i]);
        fprintf(stderr, "\tCOST = %d, VALUE = %d\n", cost, value);
    }

    printf("%d\n", best_value);

    return EXIT_SUCCESS;
}
