#include <algorithm>
#include <vector>
#include <cstdio>
#include <cstdlib>
#include <cassert>
#include <climits>
#include <ctime>
#include <queue>
using namespace std;

#define MAXN 1000
#define BIG  0x3fffffff

typedef pair <int, int> pii;

vector <int> state;
int          g[MAXN][MAXN];
int          n, niter = 1000;

int objective() {
    int total = g[ state[n-1] ][ 0 ];
    for (int i = 1; i < n; ++i)
        total += g[ state[i-1] ][ state[i] ];

    return total;
}

inline void swap(int u, int v) {
    int temp = state[u];
    state[u] = state[v];
    state[v] = temp;
}

int main(int argc, char *argv[]) {
    scanf("%d", &n);
    assert(n <= MAXN);

    if (argc > 1)
        niter = atoi(argv[1]);

    for (int i = 0; i < n; ++i)
        for (int j = 0; j < n; ++j)
            scanf("%d", &g[i][j]);

    srand(time(NULL));

    // trazenje inicijalnog stanja pohlepnim algoritmom
    queue <int> Q;
    bool        bio[MAXN] = { 0 };

    Q.push(0);
    bio[0] = true;
    state.push_back(0);

    while (!Q.empty()) {
        int u = Q.front(); Q.pop();
        int m = BIG, m_idx = -1;
        
        for (int v = 0; v < n; ++v)
            if (!bio[v] && g[u][v] < m)
                m = g[u][v], m_idx = v;

        if (m_idx == -1)
            break;

        Q.push(m_idx);
        bio[m_idx] = true;
        state.push_back(m_idx);
    }

    // hill climbing
    int cur = objective();
    while (niter--) {
        pii best = make_pair(-1, -1);

        for (int i = 1; i < n-1; ++i)
            for (int j = i+1; j < n; ++j) {
                swap(i, j);
                int p = objective();
                swap(i, j);

                if (p < cur)
                    cur = p, best = make_pair(i, j);
            }

        if (best.first == -1)
            break;

        swap(best.first, best.second);
    }

    printf("%d\n", cur);
    for (int i = 0; i < n; ++i)
        printf("%d ", state[i]+1);
    printf("\n");

    return EXIT_SUCCESS;
}
