#include <algorithm>
#include <vector>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>
#include <climits>
#include <ctime>
#include <queue>
using namespace std;

#define MAXN 1000
#define BIG  0x3fffffff

typedef pair <int, int> pii;

vector <int> state;
int          g[MAXN][MAXN];
int          n, niter = 1000;
double       T, alfa;

int objective() {
    int total = g[ state[n-1] ][ 0 ];
    for (int i = 1; i < n; ++i)
        total += g[ state[i-1] ][ state[i] ];

    return total;
}

inline int irand(int n) {
    return (int)((double)n * (rand() / (RAND_MAX + 1.0)));
}

inline void swap(int u, int v) {
    int temp = state[u];
    state[u] = state[v];
    state[v] = temp;
}

int main(int argc, char *argv[]) {
    if (argc != 4) {
        fprintf(stderr, "Koristenje: %s niter temp alfa\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    scanf("%d", &n);
    assert(n <= MAXN);

    niter = atoi(argv[1]);
    sscanf(argv[2], "%lf", &T);
    sscanf(argv[3], "%lf", &alfa);

    for (int i = 0; i < n; ++i)
        for (int j = 0; j < n; ++j)
            scanf("%d", &g[i][j]);

    srand(time(NULL));

    // trazenje inicijalnog stanja pohlepnim algoritmom
    queue <int> Q;
    bool        bio[MAXN] = { 0 };

    Q.push(0);
    bio[0] = true;
    state.push_back(0);

    while (!Q.empty()) {
        int u = Q.front(); Q.pop();
        int m = BIG, m_idx = -1;
        
        for (int v = 0; v < n; ++v)
            if (!bio[v] && g[u][v] < m)
                m = g[u][v], m_idx = v;

        if (m_idx == -1)
            break;

        Q.push(m_idx);
        bio[m_idx] = true;
        state.push_back(m_idx);
    }

    // simulated annealing
    int cur = objective(), best = cur;
    while (niter--) {
        int u = 1+irand(n-1), v = 1+irand(n-1);
        int p, ok = 0;

        swap(u, v);
        p = objective();

        fprintf(stderr, "%d (%d, %d) ", cur, u, v);
        for (int i = 0; i < n; ++i)
            fprintf(stderr, "%d ", state[i]);
        fprintf(stderr, "OBJECTIVE = %d, TEMP = %lf\n", p, T);

        swap(u, v);

        if (p < cur)
            cur = p, ok = 1;
        else {
            double prob = exp((cur-p)/T);
            double qq   = rand()/(double)RAND_MAX;

            fprintf(stderr, " idem u stanje %d s vjerojatnoscu %lf: ", p, prob);
            if (prob - qq > 10e-9)
                cur = p, ok = 1, fprintf(stderr, "DA\n");
            else fprintf(stderr, "NE\n");
        }

        best = min(best, cur);

        if (ok)
            swap(u, v);

        T *= alfa;
    }

    printf("%d\n", best);

    return EXIT_SUCCESS;
}
