#include <algorithm>
#include <vector>
#include <deque>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>
#include <climits>
#include <ctime>
#include <queue>
using namespace std;

#define MAXN 1000
#define BIG  0x3fffffff

typedef pair <int, int> pii;

vector <int> state;
deque  <pii> tabu;
int          g[MAXN][MAXN];
int          n, niter = 1000, tabu_size = 10;

int objective() {
    int total = g[ state[n-1] ][ 0 ];
    for (int i = 1; i < n; ++i)
        total += g[ state[i-1] ][ state[i] ];

    return total;
}

inline int irand(int n) {
    return (int)((double)n * (rand() / (RAND_MAX + 1.0)));
}

inline void swap(int u, int v) {
    int temp = state[u];
    state[u] = state[v];
    state[v] = temp;
}

bool na_listi(int _u, int _v) {
    int u = min(_u, _v), v = max(_u, _v);

    for (deque<pii>::iterator it = tabu.begin(); it != tabu.end(); ++it)
        if ((*it).first == u && (*it).second == v)
            return true;

    return false;
}

void dodaj(int _u, int _v) {
    int u = min(_u, _v), v = max(_u, _v);

    tabu.push_back( make_pair(u, v) );
    if ((int)tabu.size() > tabu_size)
        tabu.pop_front();
}

int main(int argc, char *argv[]) {
    if (argc != 3) {
        fprintf(stderr, "Koristenje: %s niter tabu_size\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    scanf("%d", &n);
    assert(n <= MAXN);

    niter     = atoi(argv[1]);
    tabu_size = atoi(argv[2]);

    for (int i = 0; i < n; ++i)
        for (int j = 0; j < n; ++j)
            scanf("%d", &g[i][j]);

    srand(time(NULL));

    // trazenje inicijalnog stanja pohlepnim algoritmom
    queue <int> Q;
    bool        bio[MAXN] = { 0 };

    Q.push(0);
    bio[0] = true;
    state.push_back(0);

    while (!Q.empty()) {
        int u = Q.front(); Q.pop();
        int m = BIG, m_idx = -1;
        
        for (int v = 0; v < n; ++v)
            if (!bio[v] && g[u][v] < m)
                m = g[u][v], m_idx = v;

        if (m_idx == -1)
            break;

        Q.push(m_idx);
        bio[m_idx] = true;
        state.push_back(m_idx);
    }

    // tabu search
    int cur = objective(), best = cur;
    while (niter--) {
        int u = 1+irand(n-1), v = 1+irand(n-1);
        int p;

        swap(u, v);
        p = objective();

        printf("%d (%d, %d) ", cur, u, v);
        for (int i = 0; i < n; ++i)
            printf("%d ", state[i]);
        printf("objective = %d\n", p);

        swap(u, v);

        bool nl = na_listi(state[u], state[v]);
        if (p < cur || !nl) {
            printf("  idem u stanje %d (na listi = %d)\n", p, nl);
            cur  = p;
            best = min(best, cur);

            if (!nl)
                dodaj(state[u], state[v]);
            swap(u, v);
        }

    }

    printf("%d\n", best);

    return EXIT_SUCCESS;
}
